from recipe_scrapers.ethanchlebowski import EthanChlebowski
from tests import ScraperTest


class TestEthanChlebowskiScraper(ScraperTest):

    scraper_class = EthanChlebowski

    def test_host(self):
        self.assertEqual("ethanchlebowski.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Ethan Chlebowski", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Huevos Rancheros", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(13, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://i.imgur.com/hzIuWpU.jpg", self.harvester_class.image()
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 small corn tortillas",
                "2 eggs",
                "Salsa Ranchera (recipe below)",
                "Neutral oil for toasting tortillas and cooking eggs",
                "Chopped Cilantro",
                "Cotija cheeese or Queso fresco",
                "Pickled onions",
                "Seasoned Pinto Beans (recipe below)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        test_instructions = "Heat the salsa ranchera in the microwave or a pan until warmed through.\nSet a pan over medium-high heat and add a little bit of oil. Once hot, lightly fry the corn tortillas similar to enchiladas where they are still malleable or until crisp. Drain on a paper towel. Low-calorie option - Spritz with baking spray and toast tortillas under the broiler.\nIn the same pan and oil, fry the eggs sunny side up basting them with oil as needed. Low-calorie option - Poach the eggs in water.\nPlace the cooked eggs over the tortillas and spoon the warmed sauce over the whites. Top with some crumbling cheese and cilantro. Serve with seasoned pintos beans if you would like! Slice the yolk open and mix with the sauce to enjoy."

        self.assertEqual(test_instructions, self.harvester_class.instructions())

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Huevos rancheros are one of the simplest and most common breakfasts in Mexico, all you have to do is make the salsa ranchera, pair it with some fried tortillas and eggs and it's ready.",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper


class Inspiralized(AbstractScraper):
    @classmethod
    def host(cls):
        return "inspiralized.com"

    def title(self):
        return self.schema.title()

    def author(self):
        if self.soup.find(string="Ali Maffucci"):
            return "Ali Maffucci"

    def total_time(self):
        return self.schema.total_time()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()



from recipe_scrapers.acouplecooks import ACoupleCooks
from tests import ScraperTest


# test recipe's URL
# https://www.acouplecooks.com/garlic-butter-shrimp/
class TestACoupleCooks(ScraperTest):

    scraper_class = ACoupleCooks

    def test_host(self):
        self.assertEqual("acouplecooks.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.acouplecooks.com/garlic-butter-shrimp/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "Garlic Butter Shrimp (Fast & Easy Dinner!)", self.harvester_class.title()
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Sonja Overhiser")

    def test_total_time(self):
        self.assertEqual(8, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 pound large shrimp, deveined (peeled or unpeeled)",
                "3 garlic cloves",
                "1/2 teaspoon kosher salt",
                "3 tablespoons butter",
                "2 lemon wedges",
                "Fresh cilantro or parsley, for garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "If frozen, thaw the shrimp (see the notes above).\nMince the garlic.\nPat the shrimp dry. In a medium bowl, mix the shrimp with the garlic and salt.\nIn a large skillet, heat the butter on medium high heat. Cook the shrimp for 1 to 2 minutes per side until opaque and cooked through, turning them with tongs.\nSpritz with juice of the lemon wedges and serve immediately.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import json

from ._abstract import AbstractScraper
from ._exceptions import SchemaOrgException
from ._utils import normalize_string


class AkisPetretzikis(AbstractScraper):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.recipe_json = json.loads(
            self.soup.find("script", {"id": "__NEXT_DATA__"}).get_text()
        )

    @classmethod
    def host(cls):
        return "akispetretzikis.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        make_time = self.recipe_json["props"]["pageProps"]["ssRecipe"]["data"][
            "make_time"
        ]
        bake_time = self.recipe_json["props"]["pageProps"]["ssRecipe"]["data"][
            "bake_time"
        ]
        return make_time + bake_time

    def yields(self):
        return self.recipe_json["props"]["pageProps"]["ssRecipe"]["data"]["shares"]

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredient_sections = self.recipe_json["props"]["pageProps"]["ssRecipe"][
            "data"
        ]["ingredient_sections"]
        ingredients_list = []
        if len(ingredient_sections) == 0:
            return ingredients_list
        for ingredient in ingredient_sections[0]["ingredients"]:
            parts = []
            if ingredient["quantity"]:
                parts.append(ingredient["quantity"])
            if ingredient["unit"]:
                parts.append(ingredient["unit"])
            parts.append(ingredient["title"])
            ingredients_list.append(" ".join(parts))
        return ingredients_list

    def instructions(self):
        method = self.recipe_json["props"]["pageProps"]["ssRecipe"]["data"]["method"]
        if len(method) == 0:
            return ""
        instructions_list = [
            normalize_string(step["step"]) for step in method[0]["steps"]
        ]
        return "\n".join(instructions_list)

    def ratings(self):
        average_score = self.recipe_json["props"]["pageProps"]["ssRecipe"]["data"][
            "average_score"
        ]
        return round(float(average_score), 2)

    def cuisine(self):
        try:
            return self.schema.cuisine()
        except SchemaOrgException:
            return None

    def description(self):
        return self.schema.description()

    def language(self):
        return self.recipe_json["locale"]



# mypy: allow-untyped-defs

from recipe_scrapers.forktospoon import ForkToSpoon
from tests import ScraperTest

# Based on https://forktospoon.com/air-fryer-parmesan-tomatoes/


class TestForkToSpoonScraper(ScraperTest):

    scraper_class = ForkToSpoon

    def test_host(self):
        self.assertEqual("forktospoon.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("ForkToSpoon.com", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Air Fryer Parmesan Tomatoes", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://forktospoon.com/wp-content/uploads/2021/07/Air-Fryer-Parmesan-Tomatoes-720x720.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 tomatoes, cut in half",
                "1 cup bread crumbs",
                "¼ cup parmesan cheese",
                "1 teaspoon minced garlic",
                "1 tablespoon olive oil",
                "½ teaspoon dried basil",
                "½ teaspoon dried parsley",
                "½ teaspoon dried oregano",
                "½ teaspoon dried dill",
                "1 teaspoon salt",
                "½ teaspoon black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "In a small mixing bowl, mix the bread crumbs, parmesan cheese, minced garlic, olive oil, basil, parsley, oregano, dill, salt, and black pepper. Mix well.\nSlice your tomatoes in half, and top with the breadcrumb mixture. Set into the air fryer at 330 degrees F, air fryer sitting for 6 to 10 minutes, or until the tomatoes are perfectly roasted.\nPlate, serve, and enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("American", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Air Fryer Parmesan Tomatoes are amazing! If you are looking for the perfect summer-baked tomato recipe, this one is the one, and it's so easy to make and so flavorful!",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs
import re

from ._abstract import AbstractScraper


class ProjectGezond(AbstractScraper):
    @classmethod
    def host(cls):
        return "projectgezond.nl"

    def author(self):
        return "Project Gezond"

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).text

    def category(self):
        return ", ".join(
            [
                element.text
                for element in self.soup.find(
                    "span", {"class": "meta-category"}
                ).find_all(
                    "a", {"class": lambda x: x is not None and x.startswith("category")}
                )
            ]
        )

    def total_time(self):
        time_element = self.soup.find("em", string="Bereidingstijd:").parent
        return "".join(
            [
                element.text
                for element in time_element.children
                if element.text != "Bereidingstijd:"
            ]
        ).strip()

    def yields(self):
        # Match everything in the h2 with 'Dit heb je nodig'
        # The text inside the parentheses contains the yield for the ingredients that are listed
        return re.search(
            r"Dit heb je nodig \(([^)]+)",
            self.soup.find(
                "h2", string=lambda x: x.startswith("Dit heb je nodig")
            ).text,
        ).group(1)

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients_table = self.soup.find(
            "h2", string=lambda x: x.startswith("Dit heb je nodig")
        ).next_sibling.next_sibling
        ingredients = [
            ingredient.text
            for ingredient in ingredients_table
            if ingredient.text.strip()
        ]
        return ingredients

    def instructions(self):
        instructions_table = self.soup.find(
            "h2", string=lambda x: x.startswith("Zo maak je het")
        ).next_sibling.next_sibling.next_sibling.next_sibling
        instructions = [
            instruction.text
            for instruction in instructions_table
            if instruction.text.strip()
        ]
        return "\n".join(instructions).strip()

    def ratings(self):
        # Ratings do not exist on this site
        return None

    def cuisine(self):
        # Not listed on site
        return None

    def description(self):
        # Get the recipe's content start. The recipe will start with the description until
        # we reach the instructions.
        content_start = self.soup.find("div", {"class", "entry-content"})

        description = ""
        for content_element in content_start.children:
            # If we reach this, the ingredients are listed and the description is complete
            if content_element.text.startswith("Dit heb je nodig"):
                break

            description += content_element.text

        return description.strip()



from recipe_scrapers.przepisy import Przepisy
from tests import ScraperTest

# test recipe's URL
# https://www.przepisy.pl/przepis/placki-ziemniaczane


class TestPrzepisyScraper(ScraperTest):

    scraper_class = Przepisy

    def test_host(self):
        self.assertEqual("przepisy.pl", self.harvester_class.host())

    def test_language(self):
        self.assertEqual("pl", self.harvester_class.language())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.przepisy.pl/przepis/placki-ziemniaczane",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Placki ziemniaczane", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "przepisy.pl")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "ziemniaki 1 kilogram",
                "cebula 1 sztuka",
                "jajka 2 sztuki",
                "Przyprawa w Mini kostkach Czosnek Knorr 1 sztuka",
                "Gałka muszkatołowa z Indonezji Knorr 1 szczypta",
                "sól 1 szczypta",
                "mąka 3 łyżki",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Obierz ziemniaki, zetrzyj na tarce. Odsącz masę przez sito. Zetrzyj cebulę na tarce.\nDodaj do ziemniaków cebulę, jajka, gałkę muszkatołową oraz mini kostkę Knorr.\nWymieszaj wszystko dobrze, dodaj mąkę, aby nadać masie odpowiednią konsystencję.\nRozgrzej na patelni olej, nakładaj masę łyżką. Smaż placki z obu stron na złoty brąz i od razu podawaj.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.minimalistbaker import Minimalistbaker
from tests import ScraperTest


class TestMinimalistbakerScraper(ScraperTest):

    scraper_class = Minimalistbaker

    def test_host(self):
        self.assertEqual("minimalistbaker.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://minimalistbaker.com/vegan-cashew-ricotta-cheese-soy-free-fast-easy/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Cashew Ricotta Cheese (Soy-Free, Fast, Easy!)",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Minimalist Baker")

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://minimalistbaker.com/wp-content/uploads/2020/05/Cashew-Ricotta-Cheese-sQUARE.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 \u00bc cup raw cashews",
                "1 Tbsp lemon juice",
                "1 Tbsp nutritional yeast, plus more to taste",
                "1/2 tsp garlic powder",
                "1/4-1/2 tsp sea salt ((plus more to taste))",
                "4-6 Tbsp water",
                "1/4 cup fresh chopped parsley or cilantro",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'Soak cashews in very hot water for 30 minutes to 1 hour, or overnight (or 6 hours) in cool water. Then drain, rinse, and set aside.\nAdd soaked, drained cashews to a food processor (or a high-speed blender) along with lemon juice, nutritional yeast, garlic powder, sea salt, and lesser amount of water (4 Tbsp or 60 ml as original recipe is written // adjust if altering batch size). Mix/blend, scraping down sides as needed. Then add more water 1 Tbsp (15 ml) at a time until a thick paste forms. I find I get the best texture results with a food processor, but in a pinch, a blender can work too. It just generally requires more scraping and more liquid.\nTaste and adjust flavor as needed, adding more nutritional yeast for cheesy flavor, salt to taste, lemon juice for acidity, or garlic powder for garlic flavor. Blend again to combine.\nAt this point, the "cheese" is ready to enjoy! The flavors continue to develop and thicken when chilled. Delicious on things like pizza, pasta, lasagna, salads, and more.\nBest when fresh. Store leftover nut "cheese" in the refrigerator for up to 5-7 days or in the freezer up to 1 month (let thaw at room temperature or in the refrigerator before serving).',
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from bs4 import BeautifulSoup

from ._abstract import AbstractScraper
from ._utils import normalize_string


class LeanAndGreenRecipes(AbstractScraper):
    @classmethod
    def host(cls):
        return "leanandgreenrecipes.net"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        instructions = (
            self.soup.find("div", {"class": "item-list"}).find_next("ol").find_all("li")
        )
        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        soup = BeautifulSoup(str(self.schema.cuisine()), features="html.parser")
        return soup.get_text()

    def description(self):
        descriptions = self.soup.find(
            "div", {"class": "block-field-blocknoderecipebody"}
        ).find_all("p")
        return "\n".join(
            [normalize_string(description.get_text()) for description in descriptions]
        )



from recipe_scrapers.delish import Delish
from tests import ScraperTest


class TestDelishScraper(ScraperTest):

    scraper_class = Delish

    def test_host(self):
        self.assertEqual(self.harvester_class.host(), "delish.com")

    def test_canonical_url(self):
        self.assertEqual(
            self.harvester_class.canonical_url(),
            "https://www.delish.com/cooking/recipe-ideas/recipes/a56732/pumpkin-cheesecake-roll-recipe/",
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Pumpkin Cheesecake Roll")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Lena Abraham")

    def test_total_time(self):
        self.assertEqual(self.harvester_class.total_time(), 60)

    def test_yields(self):
        self.assertEqual(self.harvester_class.yields(), "8 servings")

    def test_image(self):
        self.assertEqual(
            self.harvester_class.image(),
            "https://hips.hearstapps.com/hmg-prod/images/delish-190814-pumpkin-roll-0046-landscape-pf-1567187755.jpg",
        )

    def test_ingredients(self):
        self.assertEqual(
            self.harvester_class.ingredients(),
            [
                "Cooking spray",
                "1 c. granulated sugar",
                "3/4 c. all-purpose flour",
                "1/2 tsp. kosher salt",
                "1 tsp. baking soda",
                "1/2 tsp. pumpkin spice",
                "3 large eggs",
                "2/3 c. pumpkin puree",
                "Powdered sugar, for rolling",
                "12 oz. cream cheese, softened",
                "1 tbsp. butter, melted",
                "1 tsp. pure vanilla extract",
                "1 1/4 c. powdered sugar",
                "1/2 tsp. kosher salt",
            ],
        )

    def test_instructions(self):
        self.assertEqual(
            self.harvester_class.instructions(),
            "Preheat oven to 350°. Line a 15” x 10” jelly roll pan with parchment and grease with cooking spray. In a large bowl, combine sugar, flour, salt, baking soda, pumpkin spice, eggs, and pumpkin puree until just combined. Spread into prepared pan and bake until a toothpick inserted in center of cake comes out clean, 15 minutes.\nMeanwhile, lay out a large kitchen towel on your counter (try to use one with little to no texture) and dust with powdered sugar. When cake is done baking, flip onto kitchen towel and gently peel off parchment paper.\nStarting at a short end, gently but tightly roll cake into a log. Let cool completely.\nMeanwhile, make filling: In a large bowl, combine cream cheese, melted butter, vanilla, powdered sugar, and salt. Using a hand mixer, whip until smooth.\nWhen cake is cooled, gently unroll (it’s ok if it remains slightly curled) and spread with cream cheese filling. Roll back up and dust with more powdered sugar. Slice and serve.",
        )



from recipe_scrapers.primaledgehealth import PrimalEdgeHealth
from tests import ScraperTest


class TestPrimalEdgeHealthScraper(ScraperTest):

    scraper_class = PrimalEdgeHealth

    def test_host(self):
        self.assertEqual("primaledgehealth.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.primaledgehealth.com/no-bake-custard/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "No Bake Custard (Keto & Carnivore-Friendly)"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Jessica Haggard")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.primaledgehealth.com/wp-content/uploads/2021/02/no-bake-custard-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tablespoon grass-fed beef gelatin",
                "1¼ cups half and half",
                "1 tablespoon sweetener (optional, see note)",
                "½ teaspoon vanilla extract",
                "2 raw egg yolks",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Sprinkle the gelatin over 2 tablespoons of water in a wide, shallow dish. The more surface area to cover, the better. Set aside to thicken.\nIf sweetening: Warm the cream over low heat in a small saucepan on the stovetop. Add the sweetener and stir until it dissolves. Then stir in vanilla.If not sweetening: Add vanilla to half and half in a bowl without heating.\nWhisk egg yolks into the half and half.\nMix in the gelatin. Gently warm liquid over low heat until gelatin dissolves if needed.\nDivide into individual ramekins or use one single serving dish. Chill in the refrigerator for at least 1 hour until set. Serve cold.",
            self.harvester_class.instructions(),
        )



from responses import GET

from recipe_scrapers.goustojson import GoustoJson
from tests import ScraperTest


class TestGoustoScraper(ScraperTest):

    scraper_class = GoustoJson

    @classmethod
    def expected_requests(cls):
        yield GET, "https://www.gousto.co.uk/cookbook/recipes/malaysian-style-coconut-meat-free-chicken-pickled-cucumber", "tests/test_data/gousto.testjson"
        yield GET, "https://production-api.gousto.co.uk/cmsreadbroker/v1/recipe/malaysian-style-coconut-meat-free-chicken-pickled-cucumber", "tests/test_data/gousto.testjson"

    def test_host(self):
        self.assertEqual("gousto.co.uk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Malaysian-Style Coconut Meat-Free Chicken With Pickled Cucumber",
        )

    def test_image(self):
        self.assertEqual(
            "https://s3-eu-west-1.amazonaws.com/s3-gousto-production-media/cms/mood-image/1930--Malaysian-Coconut-Chicken--Pickled-Cucumber-1636110687600.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "50g solid creamed coconut",
                "15g fresh root ginger",
                "1 tsp ground turmeric",
                "2 shallots",
                "15ml soy sauce",
                "165g meat-free chicken bites",
                "130g basmati rice",
                "15g tamarind paste",
                "30ml rice vinegar",
                "1 red chilli",
                "1/2 cucumber",
                "3 garlic cloves",
                "25g blanched almonds",
                "1 fresh lemongrass",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Add the basmati rice and 300ml [600ml] cold water to a pot with a lid and bring to the boil over a high heatOnce boiling, reduce the heat to very low and cook, covered, for 10-12 min or until all the water has absorbed and the rice is cookedOnce cooked, remove from the heat and keep covered until serving\nWhile the rice is cooking, bash the lemongrass stalk[s] with a rolling pin, cut down the middle lengthways, remove the tough outer layers and chop the softer inner core[s] finelyPeel and roughly chop the shallots, garlic and gingerChop half of the red chilli[es] roughly, and finely slice the rest (save these for garnish!)Put everything into a food processor\nAdd the blanched almonds and half the ground turmeric (you’ll use the rest later!) to the food processor with 2 tbsp [4 tbsp] vegetable oilAdd the soy sauce and a pinch of sugarPulse until you're left with a slightly chunky paste – this is your spice paste\nBoil a kettle, then heat a large wide-based pan (preferably non-stick with a matching lid), with a drizzle of vegetable oil over a medium-high heatCut the meat-free chicken into smaller, bite-sized piecesAdd the meat-free chicken pieces to the pan and sprinkle over the remaining ground turmeric and a pinch of salt and pepperCook for 2-3 min or until warmed through and starting to brown\nWhile the meat-free chicken is cooking, cut the cucumber[s] in half lengthways and then slice finely\nAdd the sliced cucumber to a bowl with the rice vinegar, 1 tsp [2 tsp] sugar and a generous pinch of saltStir it all together and set aside until serving – this is your quick-pickled cucumber\nAdd the spice paste to the meat-free chicken and cook for 2-3 min or until fragrantOnce fragrant, add the tamarind paste with 200ml [300ml] boiled water and cook for a further 2-3 minChop the creamed coconut roughly (if required!), then add it to the pan and cook for 1 min further – this is your Malaysian-style coconut meat-free chicken\nServe the Malaysian-style coconut meat-free chicken with the cooked rice and quick-pickled cucumber to the sideGarnish with the reserved sliced chilli rounds (can't handle the heat? Go easy!)Enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5, self.harvester_class.ratings())



from recipe_scrapers.southernliving import SouthernLiving
from tests import ScraperTest


class TestSouthernLiving(ScraperTest):

    scraper_class = SouthernLiving

    def test_host(self):
        self.assertEqual("southernliving.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.southernliving.com/recipes/spicy-sausage-cheddar-kolaches-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Spicy Sausage-and-Cheddar Kolaches Recipe"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Paige Grandjean")

    def test_total_time(self):
        self.assertEqual(180, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("22 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fstatic.onecms.io%2Fwp-content%2Fuploads%2Fsites%2F24%2F2015%2F09%2F2546801_bread_02_1_0_1_0_0_0-2000.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup whole milk",
                "3/4 cup salted butter, divided",
                "1/2 cup warm water (100°F to 110°F)",
                "1 (1⁄4-oz.) envelope active dry yeast",
                "1/3 cup plus 1 tsp. granulated sugar, divided",
                "2 large eggs, beaten",
                "3/4 teaspoon salt",
                "5 1/4 - 6 cups bread flour, divided, plus more for dusting",
                "1 1/4 pounds Conecuh sausage or spicy smoked sausage",
                "4 ounces sharp Cheddar cheese, shredded (about 1 cup)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Combine whole milk and 1⁄2 cup of the butter in a small saucepan. Cook over medium-low, stirring occasionally, until butter melts, about 5 minutes. Transfer mixture to a medium bowl, and cool slightly to 100°F to 110°F, about 10 minutes.",
                    "Stir together warm water, active dry yeast, and 1 teaspoon of the granulated sugar in a small bowl. Let stand until foamy, about 5 minutes.",
                    "Combine eggs, salt, and remaining 1⁄3 cup granulated sugar in the bowl of a heavy-duty electric stand mixer fitted with a dough hook attachment. Beat on medium-low speed just until combined. Stir in milk mixture and yeast mixture. Gradually add 4 1⁄2 cups of the bread flour, beating just until incorporated. With mixer running on low speed, gradually add up to 3⁄4 cup more flour, 1⁄4 cup at a time, just until dough pulls away from sides of bowl.",
                    "Turn dough out onto a lightly floured work surface. Knead until smooth and elastic, about 10 minutes, adding up to 3⁄4 cup flour, in very small amounts, if necessary to keep dough workable. Place dough in a lightly greased bowl, turning to coat all sides. Cover and let stand in a warm place until dough doubles in size, about 1 hour.",
                    "Cut sausage into 20 to 22 (2-inch-long) straight pieces, reserving curved pieces for another use. Divide dough into 20 to 22 (2-ounce) balls (a little larger than a golf ball). Roll each ball into a 4-inch-wide circle on a lightly floured surface. (Keep remaining dough balls covered while working.) Place about 1 tablespoon of the cheese on lower third of each dough circle, and top with 1 sausage piece. Fold dough over filling, folding in sides; pinch to seal. Place kolaches, seam side down, on 2 baking sheets lined with parchment paper, leaving 1 inch between them. Keep covered while working. Cover loosely with plastic wrap, and let stand in a warm place until dough doubles in size, about 45 minutes.",
                    "Preheat oven to 375°F. Microwave remaining 1⁄4 cup butter in a small microwavable bowl on HIGH until melted, about 45 seconds. Brush kolaches with half of melted butter. Bake in preheated oven until golden brown, 14 to 16 minutes. Brush with remaining melted butter, and serve hot.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings_raises_exception(self):
        self.assertRaises(Exception, self.harvester_class.ratings)

    def test_description(self):
        self.assertEqual(
            "Kolaches are beloved in Texas, but we believe they deserve more widespread attention. Our savory spin on this classic Czech pastry is filled with a combo of smoky Conecuh sausage and sharp Cheddar cheese. Serve with coffee and a fruit salad for a simple yet satisfying breakfast. Leftover kolaches can be reheated in the microwave, but they're best when served fresh out of the oven and still warm.",
            self.harvester_class.description(),
        )



from recipe_scrapers.mybakingaddiction import MyBakingAddiction
from tests import ScraperTest


class TestMyBakingAddictionScraper(ScraperTest):

    scraper_class = MyBakingAddiction

    def test_host(self):
        self.assertEqual("mybakingaddiction.com", self.harvester_class.host())

    def test_image(self):
        self.assertEqual(
            "https://www.mybakingaddiction.com/wp-content/uploads/2016/08/slice-from-chocolate-zucchini-loaf-hero-720x720.jpg",
            self.harvester_class.image(),
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.mybakingaddiction.com/chocolate-coconut-zucchini-bread/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chocolate Zucchini Bread")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Jamie")

    def test_ratings(self):
        self.assertEqual(self.harvester_class.ratings(), 4.3)

    def test_total_time(self):
        self.assertEqual(75, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 1/2 cups all-purpose flour",
                "1/2 cup unsweetened cocoa powder",
                "1 teaspoon salt",
                "1 teaspoon baking soda",
                "1/2 teaspoon baking powder",
                "1 teaspoon ground cinnamon",
                "1 cup coconut oil, melted",
                "2/3 cup granulated sugar",
                "2/3 cup light brown sugar",
                "1/2 cup sour cream",
                "3 large eggs",
                "2 teaspoons pure vanilla extract",
                "2 1/2 cups grated zucchini",
                "1 cup semi-sweet chocolate chips",
                "3/4 cup shredded sweetened coconut",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 350°F. Spray two 8x4-inch loaf pans with baking spray and/or line with parchment paper.\nIn a medium bowl, whisk together the flour, cocoa, salt, baking soda, baking powder and cinnamon.\nIn a large bowl with an electric mixer, mix the coconut oil and sugars until combined. Mix in the sour cream. Add in the eggs and vanilla and mix until thoroughly incorporated.\nSlowly add dry ingredients to wet ingredients and mix until just combined.\nAdd in the zucchini and mix for about 1 minute, or until the batter is moistened and the zucchini is evenly incorporated into the batter. Stir in the chocolate chips and shredded coconut.\nSpread the batter into the prepared pans and bake in preheated oven for 55-60 minutes, or until a toothpick inserted into the center comes out clean.\nCool bread in pan for 30 minutes. Remove bread to a wire rack to cool completely.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.fifteenspatulas import FifteenSpatulas
from tests import ScraperTest


class TestFifteenSpatulasScraper(ScraperTest):

    scraper_class = FifteenSpatulas

    def test_host(self):
        self.assertEqual("fifteenspatulas.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.fifteenspatulas.com/orange-scented-creme-brulee/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Creme Brulee")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Joanne Ozug")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.fifteenspatulas.com/wp-content/uploads/2019/05/Orange-Creme-Brulee-Fifteen-Spatulas-16.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "zest of 4 oranges* ((about 2 tsp))",
                "3 cups heavy cream",
                "5 large egg yolks",
                "1/2 cup sugar +1 tsp for each crème brûlée",
                "1 tsp vanilla extract",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Combine the orange zest and cream in a saucepan, and let it sit in the fridge for 2 hours.\nPreheat the oven to 300 degrees F.\nWhisk together the egg yolks and 1/2 cup sugar for 1 minute, until well blended.\nHeat the orange zest cream over medium high heat until 180F, bringing it almost to a boil, but not quite. This is known as a scalding temperature. It's best to use a thermometer, but if you don't have one, you'll know the cream is hot enough when bubbles begin forming on the side, but it's not yet boiling.\nWhile whisking constantly, slowly dribble the hot cream into the egg yolk mixture, gradually over a minute.\nAdd the vanilla extract, then pour the mixture through a sieve to strain out the orange zest and any coagulated egg.\nPour the strained custard into six 4-ounce ramekins until nearly full (you may use other size ramekins, but you'll need to adjust bake time).\nPlace the ramekins in a large baking pan and add enough boiling water to come halfway up the outsides of the ramekins.\nBake for 35-40 minutes, until the creme brulees jiggle slightly when shaken, and have set. Take the ramekins out of the water bath and let cool to room temperature. Then cover the tops with plastic wrap and refrigerate until they firm up, 4-6 hours.\nWhen you’re ready to serve the creme brulee, sprinkle 1 tsp of sugar evenly on top of each one, to prepare for caramelization.\nTorch Option: Ideally, use a handheld blowtorch to quickly caramelize the tops of each creme brulee, and serve immediately. This will give you the crunchy top layer you want, while keeping the creme brulee from getting warm.\nOven broiler option: If you don't have a torch, you can use the broiler of your oven to caramelize the top. Set the oven rack as close as possible to the broiler, and preheat to high. Place the ramekins on a sheet pan, and broil for 1-2 minutes, until the top caramelizes, making sure you keep your eye on the browning (it's easier to burn the sugar using the broiler). If your oven's broiler isn't strong enough to caramelize the sugar without heating the filling, then place the ramekins back into the fridge for an hour before serving, to re-firm the custard. Serve and enjoy!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.mykitchen101 import MyKitchen101
from tests import ScraperTest


class TestMyKitchen101Scraper(ScraperTest):

    scraper_class = MyKitchen101

    def test_host(self):
        self.assertEqual("mykitchen101.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("清闲廚房 团队", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("古早味迷你烤鸡蛋糕", self.harvester_class.title())

    def test_yields(self):
        self.assertEqual("30 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://mykitchen101.com/wp-content/uploads/2020/11/mini-baked-egg-sponge-cake-mykitchen101-feature1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 个 蛋 (A级)",
                "125 克 细砂糖",
                "⅛ 茶匙 细盐",
                "115 克 普通面粉",
                "20 克 玉米淀粉",
                "30 克 溶化牛油",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "1 把烤炉预热至200°C/395°F。\n2 把蛋轻轻打散，加入盐和和糖，打至混合。\n3 把1公升的清水和400毫升的热水(热水炉取出的热水)混合在比搅拌碗大的钢盆以调出约45°C/113°F的温水。\n4 把搅拌碗浸泡在温水里，以中高速把蛋打至浓稠 (约5分钟)。(温馨提示：隔着温水打蛋糊可以缩短打发蛋糊的时间。)\n5 把普通面粉和玉米淀粉混合过筛2次。\n6 慢慢把粉类加入蛋糊里，以低速混合，再用刮刀翻拌至均匀。慢慢加入溶化牛油，用刮刀轻轻翻拌至混合。\n7 把面糊倒入裱花袋里。\n8 把迷你玛芬蛋糕模 (直径 = 4.8 cm，深 = 2.2 cm) 铺上杯子蛋糕纸托，装入面糊直到80%满。\n9 放入已预热的烤炉，以190°C/375°F 烘烤20-22分钟，直到呈金黄色。(温馨提示：不同烤炉的温度不一样，烘烤时间只供参考，可依个自的烤炉调整烘烤的时间。如果第一批烤24个蛋糕，第二批只有6个，那么第二批的烘烤时间可以缩短，只要蛋糕表面呈金黄色就可以了。)\n10 把鸡蛋糕脱模后放在铁架上至冷却。",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.littlespicejar import LittleSpiceJar
from tests import ScraperTest


class TestLittleSpiceJarScraper(ScraperTest):

    scraper_class = LittleSpiceJar

    def test_host(self):
        self.assertEqual("littlespicejar.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Easy Addicting Crab Cake Bites (Crab Cake Cups)",
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("24 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://littlespicejar.com/wp-content/uploads/2020/12/Addicting-Mini-Crab-Cake-Bites-1-scaled-720x720.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "Cooking spray, for pan",
                "1 cup panko bread crumbs",
                "½ cup parmesan cheese (shredded or grated)",
                "6 tablespoon melted butter",
                "8 ounces lump crab meat",
                "¾ brick (6 ounces) cream cheese, softened to room temp",
                "1 large egg, lightly beaten",
                "¼ cup mayonnaise",
                "½ cup sour cream",
                "½ teaspoon EACH: garlic powder AND smoked paprika",
                "1 teaspoon EACH: old bay seasoning AND lemon zest",
                "2 teaspoons lemon juice",
                "1 tablespoon chopped chives (plus more for topping)",
                "Salt and white pepper (or black)",
                "Serve with sauce (in notes)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "PANKO: Position a rack in the center of the oven and preheat the oven to 400ºF. Grease a mini muffin pan with cooking spray. Mix together the panko, parmesan cheese, and melted butter. Place roughly 2 teaspoons of the panko mixture in the bottom of each muffin cup using your fingers or the bottom of a measuring teaspoon (or tablespoon) gently pack into a tight shape.\nCRAB MIXTURE: Combine the crab meat, cream cheese, mayonnaise, egg, sour cream garlic powder, smoked paprika, old bay seasoning, lemon juice + zest, chopped chives, and ¼ teaspoon kosher salt, and ¼ teaspoon pepper. Spoon roughly 1 tablespoon of the crab mixture into each mini muffin cup.\nBAKE: Until the edges begin to turn golden, 14-18 minutes. Allow the crab cakes to cool in the pan for several minutes before attempting to remove them. You can run an offset spatula or a pairing knife along the edges to help pop out the bites. Serve crab bites warm or allow them to cool to room temperature before serving. I like to top them with more chopped chives before serving and serve with lemon wedges and prepared sauce.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.cookstr import Cookstr
from tests import ScraperTest


class TestCookstrScraper(ScraperTest):

    scraper_class = Cookstr

    def test_host(self):
        self.assertEqual("cookstr.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.Cookstr.com/recipes/chocolate-cake-nicole-axworthy",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chocolate Cake")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_total_yields_raises_exception(self):
        self.assertRaises(Exception, self.harvester_class.yields)

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 recipe Chocolate Cake Mix",
                "1/2 cup coffee or water",
                "1/2 cup almond or soy milk (vanilla flavor preferred)",
                "1/2 cup canola oil",
                "1/2 cup pure maple syrup",
                "2 tablespoons apple cider vinegar",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 350°F. Lightly grease a 9-inch cake pan with coconut oil or line a 12-cup muffin tin with paper liners.\nIn a large bowl, sift the dry cake mix ingredients using a fine-mesh sieve.\nIn a medium bowl, mix together the coffee, almond milk, oil, maple syrup, and vinegar.\nAdd the liquid ingredients to the bowl with the cake mix and whisk gently until there are no large clumps remaining.\nPour the batter into the prepared pan. Bake for 22 to 27 minutes in the cake pan or 20 to 25 minutes in the muffin tin. The cake/cupcakes can be stored in an airtight container in the fridge for up to 5 days or frozen for 2 to 3 months.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.weightwatchers import WeightWatchers
from tests import ScraperTest


class TestWeightwatchersScraper(ScraperTest):

    # Test-Url:
    # https://cmx.weightwatchers.de/details/WWRECIPE:5667ab72a29713e4335bb342

    scraper_class = WeightWatchers

    def test_host(self):
        self.assertEqual("www.weightwatchers.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("WeightWatchers", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Würstchengulasch mit Nudeln")

    def test_category(self):
        self.assertEqual("WeightWatchers", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(0, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(25, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cmx.weightwatchers.com/assets-proxy/weight-watchers/image/upload/t_WINE_EXTRALARGE/i34cskr1hxegmxqukawd.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Stück Geflügelwürstchen",
                "1 Stück, klein Zwiebel/n",
                "200 g Champignons, frisch; braun",
                "120 g Nudeln, trocken, jede Sorte; Spiralnudeln",
                "1 Prise(n) Salz/Jodsalz",
                "2 TL Pflanzenöl, Rapsöl/Sonnenblumenöl",
                "400 g Tomaten, passiert",
                "1 Prise(n) Pfeffer",
                "1⁄2 TL Paprikapulver",
            ],
            self.harvester_class.ingredients(),
        )

    def test_ingredients_count(self):
        self.assertEqual(9, len(self.harvester_class.ingredients()))

    def test_instructions(self):
        self.assertEqual(
            "Würstchen in Scheiben schneiden. Zwiebel schälen und würfeln. Champignons trocken abreiben und vierteln. Nudeln nach Packungsanweisung in Salzwasser garen.\nÖl in einem Topf erhitzen und Zwiebelwürfel darin andünsten. Würstchenscheiben und Champignonviertel zufügen und ca. 3 Minuten anbraten. Mit Tomaten ablöschen, aufkochen und ca. 5 Minuten köcheln lassen. Würstchengulasch mit Salz, Pfeffer und Paprikapulver würzen. Nudeln abgießen, untermischen und in einer Frischhaltedose transportieren. Würstchengulasch erwärmen und servieren.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "18 Uhr und alle haben Hunger? Dann koche rasch das Würstchengulasch und alle sind happy.",
            self.harvester_class.description(),
        )

    def test_difficulty(self):
        self.assertEqual("Leicht", self.harvester_class.difficulty())

    def test_nutrients(self):
        expected_nutrients = {
            "personal points": "earn 12 personal points",
            "positive points": "+2 Punkte von 2 Portion(en) Gemüse",
        }
        self.assertEqual(self.harvester_class.nutrients(), expected_nutrients)



from recipe_scrapers.hundredandonecookbooks import HundredAndOneCookbooks
from tests import ScraperTest


class TestHundredAndOneCookbooksScraper(ScraperTest):

    scraper_class = HundredAndOneCookbooks

    def test_host(self):
        self.assertEqual("101cookbooks.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.101cookbooks.com/blood-orange-gin-sparkler/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Blood Orange Gin Sparkler")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Heidi Swanson")

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.101cookbooks.com/gin_sparkler.jpg?w=1200&auto=format",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups / 480 ml water",
                "1 cup / 6.5 oz / 185 g sugar",
                "4 tablespoons (~2 sprigs-worth) fresh rosemary leaves",
                "1 bay leaf (optional)",
                "blood oranges",
                "gin",
                "ice cubes",
                "tonic water (or sparkling water)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Combine the water, sugar, rosemary, and bay in a small saucepan over medium heat. Bring to a simmer for 3-5 minutes, or long enough for the sugar to dissolve, stirring occasionally. Remove from the heat and let infuse for 10 minutes. Strain into a jar to cool completely.\nIn the meantime, juice and strain your oranges, you'll need 3 tablespoons / 1.5 ounces of juice for each drink.\nTo make each drink you're going to combine equal parts gin, juice, and tonic water with a bit of syrup and ice. So, its 3 tablespoons / 1 1/2 oz gin, 3 tablespoons / 1 1/2 oz freshly squeezed blood orange juice, and 1-2 teaspoons of the rosemary syrup in each tall glass (I used kolsch glasses here). Stir to combine, fill each glass 2/3 full with ice and top off with 3 tablespoons / 1 1/2 oz tonic water. Stir again and you're set.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.coop import Coop
from tests import ScraperTest


class TestCoopScraper(ScraperTest):

    scraper_class = Coop

    def test_host(self):
        self.assertEqual("coop.se", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(
            "Sara Begner och Ulrika Brydling", self.harvester_class.author()
        )

    def test_title(self):
        self.assertEqual(
            "Grillat kött med lime- och chilismör", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Huvudrätt", self.harvester_class.category())

    def test_cook_time(self):
        self.assertEqual(15, self.harvester_class.cook_time())

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://res.cloudinary.com/coopsverige/image/upload/w_1200,h_1200/v1651049395/cloud/251336.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "600.0 g flapsteak, flankstek, entrecôte eller mörad högrev",
                "1.0 tsk flingsalt",
                "100.0 g rumsvarmt smör",
                "1.0 röd chili, skivad",
                "2.0 lime",
                "4.0 hjärtsalladshuvud",
                "0.0 flingsalt",
                "0.0 grovmalen svartpeppar",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Tänd grillen med kol/briketter på ena sidan. Salta köttet. Vispa smöret fluffigt och smaksätt med chili och finrivet skal av 1 lime.\nVid fin glöd, grilla köttet direkt över värmen ca 1–2 min på varje sida. Köttet ska få fin yta och en innertemp på 52–55°. Låt vila 5 min.\nDela lime och hjärtsallad och grilla ca 5 min. Skär köttet i skivor, strö över flingsalt och svartpeppar och servera med hjärtsallad, smör och pressad lime.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.71, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Mer Smak", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Grillat kött som toppas med ett smakrikt smör på chili och pressad lime. Servera med grillad hjärtsallad.",
            self.harvester_class.description(),
        )



from recipe_scrapers.headbangerskitchen import HeadbangersKitchen
from tests import ScraperTest


class TestHeadbangersKitchenScraper(ScraperTest):

    scraper_class = HeadbangersKitchen

    def test_host(self):
        self.assertEqual("headbangerskitchen.com", self.harvester_class.host())

    # def test_canonical_url(self):
    #     self.assertEqual(
    #         'https://headbangerskitchen.com/recipe/keto-omelet-indian-style/',
    #         self.harvester_class.canonical_url(),
    #     )

    def test_author(self):
        self.assertEqual("Sahil Makhija", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Keto Omelet (Indian Style)", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(10, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://headbangerskitchen.com/wp-content/uploads/2020/11/KETOMASALAOMELET-Vertical.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 Eggs",
                "20 grams Cheese",
                "20 grams Red onion",
                "1 Tbsp Heavy Whipping Cream ( Order online )",
                "1/2 Tsp Tumeric ( Order online )",
                "1/2 Tsp Kashmiri Red Chilli Powder ( Order online )",
                "1 Tbsp Ghee",
                "salt and pepper to taste",
                "1 Tsp Coriander",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Crack the 3 eggs into a bowl and add in the chopped onion, coriander, salt, pepper, tumeric, chilli powder and heavy cream and beat well.\nHeat the ghee in a frying pan and once melted add in the beaten eggs. Grate in the cheese and cover and cook for about 4 minutes. Fold the egg and finish cooking.\nServe with a side of salad.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.33, self.harvester_class.ratings())



from recipe_scrapers.bettybossi import BettyBossi
from tests import ScraperTest


class TestBettyBossiScraper(ScraperTest):

    scraper_class = BettyBossi

    def test_host(self):
        self.assertEqual("bettybossi.ch", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Betty Bossi", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Minicheesecakes", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Dessert", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(90, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.bettybossi.ch/rdbimg/bb_blub160501_0070a/bb_blub160501_0070a_r01_v005_x0010.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "120 g de biscuits au beurre (p. ex. petits-beurre)",
                "40 g de beurre, fondu, refroidi",
                "250 g de fromage frais double crème (p. ex. Philadelphia)",
                "250 g de mascarpone",
                "150 g de séré demi-gras",
                "3 œufs, battus",
                "120 g de sucre",
                "1 sachet de sucre vanillé",
                "1 citron bio, zeste râpé et 1 c.s. de jus",
                "2 c.s. de liqueur au cassis ou de sirop de cassis",
                "2 c.s. de farine",
                "250 g de myrtilles surgelées, dégelées",
                "2 c.s. de sucre",
                "1 c.s. de jus de citron",
                "2 c.s. de liqueur au cassis ou de sirop de cassis",
                "1 citron bio, zeste râpé",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Écraser les biscuits dans un sachet en plastique à l’aide d’un rouleau à pâtisserie, mélanger avec le beurre. Répartir sur le fond du moule préparé, bien tasser avec le dos d’une cuillère ou le fond d’un verre, réserver au frais.\nBien mélanger au fouet le fromage frais, le mascarpone et le séré. Incorporer les œufs et tous les ingrédients, farine comprise.\nBien égoutter les myrtilles et mixer les myrtilles avec le reste des ingrédients, étaler sur le fond de petits-beurre, répartir dessus l’appareil au fromage frais.\nCuisson: env. 50 min dans la moitié inférieure du four préchauffé à 180° C. Laisser tiédir ensuite env. 1 h dans le four éteint, en maintenant la porte entrouverte avec le manche d’une spatule en bois. Retirer, laisser refroidir sur une grille. Ôter le bord du moule.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(
            "Dessert, Patisserie salée et sucrée", self.harvester_class.cuisine()
        )

    def test_description(self):
        self.assertEqual(
            "Ces larges biscuits cachent une crème composée de myrtilles avec un mélange de fromage frais, de séré et de mascarpone et n’attendent que vous pour les couper!",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class CookieAndKate(AbstractScraper):
    @classmethod
    def host(cls):
        return "cookieandkate.com"

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).get_text()

    def total_time(self):
        return get_minutes(
            self.soup.find("span", {"class": "tasty-recipes-total-time"})
        )

    def yields(self):
        yields = self.soup.find("span", {"class": "tasty-recipes-yield"}).get_text()

        return get_yields("{} servings".format(yields))

    def ingredients(self):
        ingredients = self.soup.find(
            "div", {"class": "tasty-recipe-ingredients"}
        ).find_all("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find(
            "div", {"class": "tasty-recipe-instructions"}
        ).find_all("li")

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        return round(float(self.soup.find("span", {"class": "average"}).get_text()), 2)



from recipe_scrapers.geniuskitchen import GeniusKitchen
from tests import ScraperTest


class TestGeniusKitchenScraper(ScraperTest):

    scraper_class = GeniusKitchen

    def test_host(self):
        self.assertEqual("geniuskitchen.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.geniuskitchen.com/recipe/quiche-lorraine-cups-19170",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Quiche Lorraine Cups")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Bergy")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "12 cooked crepes (, see All Purpose Dinner Crepes Batter)",
                "4 slices bacon, cooked crisp &,crumbled",
                "1 cup swiss cheese, grated",
                "2 tablespoons flour",
                "1/4 teaspoon salt",
                "2 eggs",
                "1 cup milk",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Lightly grease a 12 muffin pan or 12 custard cups.\nLine each with a crepe, fluting them.\nSprinkle bacon into the crepes.\nDivide the cheese between the crepes.\nMix together the flour, salt.\nMix the beaten eggs and milk, add to the flour.\nBlend well and pour into the crepes on top of the cheese.\nBake in 350F oven for 15-20 minutes or until firm.\nCool 5 minutes before removing from pan.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.lecremedelacrumb import LeCremeDeLaCrumb
from tests import ScraperTest


class TestLeCremeDeLaCrumbScraper(ScraperTest):

    scraper_class = LeCremeDeLaCrumb

    def test_host(self):
        self.assertEqual("lecremedelacrumb.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.lecremedelacrumb.com/instant-pot-shredded-chicken-tacos/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Instant Pot Shredded Chicken Tacos"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Tiffany")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_image(self):
        self.assertEqual(
            "https://www.lecremedelacrumb.com/wp-content/uploads/2019/01/instant-pot-shredded-chicken-tacos-5.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3-4 medium to large boneless skinless chicken breasts",
                "1 cup chicken broth or water",
                "1 teaspoon salt",
                "1 teaspoon ground cumin",
                "1 teaspoon chili powder",
                "1 teaspoon garlic powder",
                "1 15-ounce can fire roasted tomatoes",
                "corn or flour taco-size tortillas",
                "avocado, tomatoes, cheese, cilantro, sour cream (for serving)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In your instant pot/pressure cooker combine chicken, broth or water, cumin, chili powder, garlic powder, and fire roasted tomatoes (with liquid).\nCover and set to PRESSURE COOK or MANUAL for 20 minutes. (30 minutes if using frozen chicken breasts)\nDo a quick release (turn vent knob to the VENT position and allow to de-pressurize until the float valve drops down) then uncover and shred chicken with two forks.\nServe chicken in tortillas topped with cheese, tomatoes, avocado, cilantro, or any other favorite toppings.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.64, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import normalize_string


class MadeWithLau(AbstractScraper):
    @classmethod
    def host(cls):
        return "madewithlau.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def description(self):
        return self.schema.description()

    def cook_time(self):
        return self.schema.cook_time()

    def prep_time(self):
        return self.schema.prep_time()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        # collect headers
        headers = [
            header.find("h4")
            for header in self.soup.findAll(
                "div",
                {"class", re.compile("summary-step_step-label.*")},
            )
        ]

        # collect steps associated with each header
        step_sets = self.soup.findAll(
            "div",
            {"class", re.compile("summary-step_step__.*")},
        )

        # merge headers and steps
        instructions = [
            p
            for pair in zip(headers, step_sets)
            for p in [
                pair[0],
                *(pair[1].find("div") or pair[1]),
            ]
        ]

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def cuisine(self):
        return self.schema.cuisine()

    def category(self):
        return self.schema.category()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class Dr(AbstractScraper):
    @classmethod
    def host(cls):
        return "dr.dk"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def language(self):
        meta_language = self.soup.find(
            "meta",
            attrs={"name": lambda x: x and x.lower() == "language", "content": True},
        )

        return meta_language.get("content")

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()



# mypy: disallow_untyped_defs=False
from html import unescape

from ._abstract import AbstractScraper
from ._schemaorg import SchemaOrg


class KitchenStories(AbstractScraper):
    @classmethod
    def host(cls):
        return "kitchenstories.com"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Workaround: kitchenstories has some unusual HTML escaping going on
        # within their on-page schema.org metadata.  Retrieve it and unescape
        # the content so that we can retrieve values from it.
        # Ref: https://github.com/hhursev/recipe-scrapers/issues/562
        schema_data = self.soup.find("script", {"type": "application/ld+json"})
        schema_data = unescape(str(schema_data))
        self.schema = SchemaOrg(schema_data)

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def cook_time(self):
        return self.schema.cook_time()

    def prep_time(self):
        return self.schema.prep_time()

    def image(self):
        return self.schema.image()

    def nutrients(self):
        return self.schema.nutrients()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def author(self):
        return self.schema.author()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.springlane import Springlane
from tests import ScraperTest


class TestSpringlaneScraper(ScraperTest):

    scraper_class = Springlane

    def test_host(self):
        self.assertEqual("springlane.de", self.harvester_class.host())

    def test_site_name(self):
        self.assertEqual("Springlane", self.harvester_class.site_name())

    def test_language(self):
        self.assertEqual("de-DE", self.harvester_class.language())

    def test_author(self):
        self.assertEqual("Springlane", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Tiramisu ohne Ei und ohne Alkohol – Dolci für alle!",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Awareness,Dessert,Espressomaschinen,Recipe,Sweet Treats",
            self.harvester_class.category(),
        )

    def test_cuisine(self):
        self.assertEqual("Italien", self.harvester_class.cuisine())

    def test_total_time(self):
        self.assertEqual(260, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(240, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(20, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.springlane.de/magazin/wp-content/uploads/2017/01/Tiramisu-ohne-Ei-und-ohne-Amaretto_15.jpg",
            self.harvester_class.image(),
        )

    def test_nutrients(self):
        self.assertEqual({}, self.harvester_class.nutrients())

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 g Mascarpone",
                "400 ml Sahne",
                "100 g Puderzucker",
                "150 ml Espresso",
                "280 g Löffelbiskuit",
                "Kakaopulver",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Sahne steif schlagen und mit Mascarpone und Puderzucker verr\u00fchren. Espresso kochen und abk\u00fchlen lassen.\n"
            + "H\u00e4lfte der L\u00f6ffelbiskuits in die Auflaufform geben. 75 ml Espresso dar\u00fcbertr\u00e4ufeln. "
            + "H\u00e4lfte der Creme auf den L\u00f6ffelbiskuits verstreichen. "
            + "Restliche L\u00f6ffelbiskuits, Espresso und Creme daraufgeben, mindestens 4 Stunden kaltstellen. Vor dem Servieren mit Kakaopulver best\u00e4uben.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.vanillaandbean import VanillaAndBean
from tests import ScraperTest


class TestVanillaAndBeanScraper(ScraperTest):

    scraper_class = VanillaAndBean

    def test_host(self):
        self.assertEqual("vanillaandbean.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://vanillaandbean.com/maple-oat-sourdough-sandwich-bread/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Maple Oat Sourdough Sandwich Bread"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Traci York | Vanilla And Bean")

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://vanillaandbean.com/wp-content/uploads/2020/06/aaasourdough-20.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 C (100g) Whole Rolled Oats (plus 2 Tbs for the top)",
                "1/2 C (105g) Water (room temperature)",
                "3/4 C (150g) Sourdough Starter (100% hydration, previously feed and bubbly)",
                "1 C (230g) Milk ((nut milk or whole dairy milk) I use 1/2 diluted unsweetened cashew milk - 80F (26C))",
                "1/4 C (80g) Maple Syrup",
                "1 Tbs (14g) Olive Oil ( + more for oiling the pan)",
                "1/2 C + 1 Tbs (100g) Whole Wheat Bread Flour ((see note**) or whole wheat flour)",
                "2 C + 3 1/2 Tbs (300g) Bread Flour",
                "1 1/2 tsp (10g) Fine Sea Salt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'For the Soaker:\nAdd the oats to a small bowl and pour the water over them. The water wont fully cover the oats - that\'s okay. Set aside.\nFor the Dough:\nMix the Dough: In the evening, whisk the starter, milk, maple syrup and oil together in a large bowl with a fork. Add the flours and salt. First, mix with a fork, the dough will be shaggy. Then mix by hand, mixing, folding and pushing the dough until the flour is fully incorporated and no dry bits are present. It will seem dry at first, but the more you work the dough, the more hydrated it will become. The dough will feel stiff and it will stick to your fingers as you go. Do this for about 3-4 minutes. Use the fork to scrape off the dough on your fingers as much as you can. Cover bowl with a damp tea towel. Set a timer for for 30 minutes and allow the dough to rest for 30 minutes. Now is a good time to feed/refresh your starter.\nIncorporate the Soaker: After the dough has rested, mix in the soaker. To do this, dump the soaker on top of the dough and begin kneading and folding the dough over and on top of itself, incorporating the ingredients of the soaker. Do this for about 2-3 minutes or until the soaker is evenly distributed in the dough. It will seem like a lot of added bits, but as you work the dough and as the dough develops through fermentation, the soaker will incorporate. The dough will be stiff, wet and sticky. Cover bowl with a damp tea towel, set a timer for 30 minutes and allow the dough to rest for 30 minutes.\nFold the Dough: After the dough has rested, fold the dough. To do this, grab a portion of the dough while it remains in the bowl, stretch it up and fold it over, pressing your fingertips into the center of the dough. You\'ll notice the dough is a little less stiff and more workable at this point. Repeat, until you\'ve worked your way around the dough. This is the first fold, and you can stop here and begin bulk fermentation, but If time permits, and optimally, you\'ll want to repeat this fold two more times as it improves the final dough\'s structure and crumb. Allow for about 30 between the second and third fold.\nBulk Fermentation (first rise): After the last fold, cover the bowl with a damp towel and allow to rise overnight at room temperature (see note* on optionally retarding the dough). This will take about 8-10 hours at 70F (21C), but in a cooler kitchen the dough can take up to 12-14 hours to rise - this is the norm in my chilly kitchen 66F (18C). Use the time as a guide and not a determining factor. The dough is ready when it no longer looks dense, is jiggly when the bowl is shaken, and has about doubled in size.\nShape the Dough: In the morning, and with damp fingertips, coax the dough into a floured work-surface. With moist fingers, take a portion of the dough, gently stretch it towards you and fold it over towards the center, pressing it down gently. Repeat this process until you work the dough all the way around the dough. Using a bench scraper flip the dough over, cover with a tea towel rest the dough for 15-20 minutes. While the dough rests, throughly coat the inside of a 9x5" (23cm X 12cm) loaf pan with olive or coconut oil. Set aside.Flour the work surface and using a bench scraper flip the dough back over. Use the photos in the blog post to help guide shaping. Gently shape the dough into a rough rectangle (sort of a football/oblong shape if you can) by dimpling the dough (think focaccia), to roughly 6" wide and 8-10" long. Fold the dough lengthwise 2/3 over onto itself and crease it firmly using the pinky side of your hand. Fold the 1/3 piece left towards the crease and pinch the seam closed. Fold in the ends, coming in about an 3/4" - 1" and pinch the seam closed. Flip the bread over and rock it back and fourth a bit and gently fluff the ends in. Cover with a tea towel and allow the dough to rest for about 5-10 minutes. Scrape your work surface area of any excess flour and very lightly spritz it with water. Place the dough on the spritz of water (seam side down) and using lightly floured hands, cup the back of the dough and gently pull the it towards you until the surface is taught. Gently "fluff" the ends in again if needed to fit into the loaf pan.Once the surface is taut and with one swift move, use the bench scraper to scoop the dough up and, place the dough into the prepared loaf pan. If there\'s any misshapen edges that need tucking in, use a moist rubber spatula to gently finesse and shape the edges of the dough down into the pan.\nProof the Dough (second rise): Cover the pan with a damp tea towel and allow the dough to rise until the dough rises to about 3/4" - 1" (2cm - 2.54cm) above the lip of the pan at the center of the dough. The amount of time will vary depending on ambient temperature. At 70F (21C) proofing takes about an hour. In my chilly kitchen, about two hours.Set an oven rack in the center of the oven. Preheat your oven to 450F (232C).\nBake the Dough: Lightly spritz the top of your loaf with water and sprinkle 1-2 Tbs of Rolled oats over the top. Place the loaf into the oven and then, reduce the oven temperature to 400F (204C). Bake for 45-55 minutes, rotating and tenting the loaf at about 25-30 minutes so it doesn\'t get too dark. The bread is ready when the internal temperature of the loaf is between 190F - 205F (87C - 96C). When done, transfer to a wire rack. Cool for 10 minutes in the pan, then transfer to a cooling rack. Cool 1 hour before slicing.\nStorage Notes: Sourdough is best consumed on the same day it\'s baked, but it lasts for 3-4 days stored at room temperature. Store at room temperature cut side down.This bread freezes beautifully. Store baked bread in a freezer bag either whole or sliced for up to two weeks. Thaw at room temperature. This bread can go straight from freezer to toaster to make the most delicious toast!',
            self.harvester_class.instructions(),
        )



from recipe_scrapers.streetkitchen import StreetKitchen
from tests import ScraperTest


class TestStreetKitchenScraper(ScraperTest):

    scraper_class = StreetKitchen

    def test_host(self):
        self.assertEqual("streetkitchen.hu", self.harvester_class.host())

    def test_language(self):
        self.assertEqual("hu", self.harvester_class.language())

    def test_canonical_url(self):
        self.assertEqual(
            "https://streetkitchen.hu/hust-hussal/kapros-tejfolos-sajtos-csirke-rizzsel/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Kapros tejfölös sajtos csirke rizzsel"
        )

    def test_total_time(self):
        self.assertEqual(None, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertListEqual(
            [
                "250 g csirkemell",
                "só, bors",
                "2 db tojás",
                "100 g liszt",
                "150 g trappista sajt",
                "300 g tejföl",
                "50 ml víz",
                "1 csokor friss kapor",
                "2 gerezd fokhagyma",
                "100 g rizs",
                "200 ml meleg víz",
                "1 tk só",
                "1 ek olaj",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "A csirkemellet felszeleteljük, sózzuk, borsozzuk és bebundázzuk: először lisztbe, majd tojásba és a végén zsemlemorzsa helyett, finomra reszelt trappista sajtba forgatjuk és egy kb. 20×10-es tepsibe rakjuk.\n"
            + "A tejfölt kikeverjük a vízzel, sózzuk, borsozzuk, belereszeljük a fokhagymát, megszórjuk a felaprított kaporral és alaposan elkeverjük. Ráöntjük a húsra és 180 fokra előmelegített sütőben 15 percig sütjük. Ezután felvesszük 200 fokra a sütő hőmérsékletét és addig sütjük amíg kicsit aranybarna nem lesz a teteje.\n"
            + "A rizst az olajon 2 percig pirítjuk. Felöntjük forró vízzel, megsózzukk és lefedve felforraljuk. Ezután alacsony lángon készre főzzük. A végén a fedőt még 5 percig rajta hagyjuk. A tejfölös sajtos csirkét  a rizzsel tálaljuk.",
            self.harvester_class.instructions(),
        )

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://485744-1530733-raikfcquaxqncofqfm.stackpathdns.com/wp-content/uploads/2019/03/kapros-tejfolos-csirke-talalasa.jpg",
            self.harvester_class.image(),
        )



from recipe_scrapers.homechef import HomeChef
from tests import ScraperTest


class TestHomeChefScraper(ScraperTest):

    scraper_class = HomeChef

    def test_host(self):
        self.assertEqual("homechef.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Chef\n\nPatrick Le Beau", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Prosciutto and Mushroom Carbonara with Asiago and peas",
            self.harvester_class.title(),
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://homechef.imgix.net/https%3A%2F%2Fasset.homechef.com%2Fuploads%2Fmeal%2Fplated%2F18612%2F5846ProsciuttoandAsiagoCarbonara_Ecomm__1_of_1_-7e576bca3e38185a1acfd06570476d88-7e576bca3e38185a1acfd06570476d88.jpg?ixlib=rails-1.1.0&w=600&auto=format&s=8ffa167fc61e45d14b773a3e06cddaf4",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 oz. Peas",
                "2 oz. Prosciutto",
                "Info 1 oz. Shredded Asiago Cheese",
                "Info 2 oz. Sour Cream",
                "Info ⅓ oz. Butter",
                "1 Lemon",
                "4 oz. Cremini Mushrooms",
                "Info 6 oz. Linguine",
                "2 Garlic Cloves",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Cook the Pasta\n\nOnce water in medium pot is boiling, add pasta and cook until al dente, 10-12 minutes.\n\nReserve 1/2 cup pasta cooking water. Drain pasta in a colander and set aside.\n\nWhile pasta cooks, prepare ingredients.\nPrepare the Ingredients\n\nCut mushrooms into 1/4" slices.\n\nHalve and juice lemon.\n\nMince garlic.\nCrisp the Prosciutto\n\nRemove prosciutto from refrigerator. Line a plate with a paper towel.\n\nPlace a large non-stick pan over medium heat and add 1/2 tsp. olive oil. Working in batches, add prosciutto to hot pan in a single layer. Cook until crispy, 1-2 minutes per side.\n\nTransfer prosciutto to towel-lined plate. When cool enough to handle, break into bite-sized pieces. Reserve pan; no need to wipe clean.\nCook the Mushrooms\n\nReturn pan used to crisp prosciutto to medium-high heat and add 1 tsp. olive oil.\n\nAdd mushrooms and a pinch of salt and pepper to hot pan. Stir occasionally until beginning to brown, 3-4 minutes.\n\nAdd garlic and stir constantly until aromatic, 30-60 seconds.\nMake Sauce and Finish Dish\n\nStir pasta, butter, a pinch of salt, and half the reserved pasta cooking water into hot pan until butter is melted. Stir in peas and half the cheese (reserve remaining for garnish) until cheese melts, 30-60 seconds.\n\nRemove from burner and stir in sour cream, 2 tsp. lemon juice, half the prosciutto (reserve remaining for garnish), 1/4 tsp. salt, and a pinch of pepper. If sauce is too thick, add additional pasta cooking water, 1 Tbsp. at a time, until desired consistency is reached.\n\nPlate dish as pictured on front of card, garnishing with remaining prosciutto and remaining cheese. Bon appétit!""",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            """This meal could go by another name: “A cured meat and umami bomb of flavor, all tangled with pasta.” But that's too long a name to go on the recipe card and besides, you already know what we mean when we say “prosciutto” and “mushrooms.” But this pasta also has a creamy lemon-y sauce, not to mention butter and cheese… now we're talking paragraphs, not meal titles.""",
            self.harvester_class.description(),
        )



from recipe_scrapers.ambitiouskitchen import AmbitiousKitchen
from tests import ScraperTest


class TestAmbitiousKitchenScraper(ScraperTest):

    scraper_class = AmbitiousKitchen

    def test_host(self):
        self.assertEqual("ambitiouskitchen.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(
            "Monique of AmbitiousKitchen.com", self.harvester_class.author()
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.ambitiouskitchen.com/vegetarian-spinach-pumpkin-lasagna/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Vegetarian Spinach Pumpkin Lasagna"
        )

    def test_total_time(self):
        self.assertEqual(70, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.ambitiouskitchen.com/wp-content/uploads/2019/10/Vegetarian-Spinach-Pumpkin-Lasagna-5sq.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "For the noodles:",
                "10 lasagna noodles",
                "For the ricotta mixture:",
                "1/2 tablespoon olive oil",
                "6 ounces spinach (from 1 bag spinach)",
                "1 (15 ounce) container part skim ricotta",
                "1 egg",
                "½ teaspoon garlic powder",
                "1/2 teaspoon salt",
                "Freshly ground black pepper",
                "For the pumpkin layer:",
                "2 (15 ounce) cans pumpkin puree",
                "½ cup milk (I like unsweetened almond milk, but any milk will work)",
                "¼ teaspoon cinnamon",
                "¼ teaspoon nutmeg",
                "¼ teaspoon ginger",
                "¼ teaspoon allspice",
                "3/4 teaspoon salt",
                "Freshly ground black pepper",
                "For the layers:",
                "3 cups shredded mozzarella cheese, divided (approximately 12 ounces)",
                "1 cup grated parmesan cheese, divided",
                "To garnish:",
                "Fresh chopped parsley or small sage leaves",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 400 degrees F. Grease a 9x13 inch baking pan with nonstick cooking spray.\nBring a large pot of water to a boil. Cook the lasagna noodles for 5-6 minutes, then drain. Immediately lay the noodles flat on an oiled baking sheet or cutting board so you can easily assemble the lasagna when ready. Another option is to soak the lasagna noodles in very warm (hot) water for 20-30 minutes if you do not want to boil them. (You can also use no cook lasagna noodles, but they aren't my favorite!)\nWhile the lasagna noodles are boiling, cook the spinach: add ½ tablespoon olive oil to a medium pan or skillet and place over medium heat. Add spinach, season with a little salt and pepper and cook until spinach wilts down. Add to a medium bowl and allow to cool for a minute or two.\nIn the small bowl, add the spinach, ricotta, egg, garlic powder, salt and pepper. Set aside.\nNext make the pumpkin mixture: add pumpkin, milk, cinnamon, nutmeg, ginger, allspice, salt and pepper to a large bowl. Mix to combine.\nTo assemble the lasagna, spread 1 heaping cup of pumpkin mixture over the bottom of the baking dish. Place 5 of the cooked lasagna noodles on top laying 4 vertically and 1 horizontally. Spread half of the spinach-ricotta cheese mixture on top of the noodles, then top with ¾ cup shredded mozzarella.\nNext, add 1 heaping cup of the pumpkin mixture on top of the mozzarella and then sprinkle with 1/2 cup of parmesan cheese.\nRepeat layers once more: adding remaining noodles, remaining spinach-ricotta mixture, ¾ cup shredded mozzarella, then top with any remaining pumpkin and ½ cup parmesan cheese. Finally, top with remaining 1 ½ cups shredded mozzarella cheese.\nCover with foil and bake covered for 25 minutes. Remove foil and bake another 15-20 minutes until cheese starts to brown just a bit. Garnish with extra parmesan and either chopped sage or parsley. Cool for 15 minutes before cutting and serving. Serves 12.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



# mypy: allow_untyped_defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class AmazingRibs(AbstractScraper):
    @classmethod
    def host(cls):
        return "amazingribs.com"

    def author(self):
        author_container = self.soup.find("p", {"class": "author-attribution"})
        if author_container and author_container.find("a"):
            return normalize_string(author_container.find("a").text)
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()



from recipe_scrapers.sweetpeasandsaffron import SweetPeasAndSaffron
from tests import ScraperTest


class TestSweetPeasAndSaffron(ScraperTest):

    scraper_class = SweetPeasAndSaffron

    def test_host(self):
        self.assertEqual("sweetpeasandsaffron.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://sweetpeasandsaffron.com/healthy-steel-cut-oats-recipes/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "7 Healthy Steel Cut Oats Recipes", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual("25", self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.statically.io/img/sweetpeasandsaffron.com/wp-content/uploads/2017/05/steel-cut-oats-7-ways-TEXT-150x150.jpg?quality=100&f=auto",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "butter 1 tablespoon",
                "steel cut oats 1 cup",
                "Instant Pot or 4 (slow cooker, stove-top) cups water 3 teaspoon",
                "vanilla extract 1 teaspoon",
                "cinnamon 1 tablespoons",
                "After cooking: 2 cup",
                "maple syrup 1 cup",
                "large apple 1 teaspoon",
                "unsweetened apple sauce 1/3 teaspoon",
                "chopped pecans 1/4 teaspoon",
                "ground nutmeg 1/4 cup",
                "ground cloves 1 tablespoons",
                "vanilla extract 1 cups",
                "pumpkin puree 2 tablespoon",
                "pumpkin seeds/pepitas 1-2 tablespoons",
                "zest from 1 lemon 1 cups",
                "blueberries 2 teaspoon",
                "chia seeds 1-2 tablespoon",
                "cocoa powder 1 cups",
                "peanut butter 1 cup",
                "berries 1-2 cup",
                "chia seeds 1/4 cup",
                "coconut extract 1 teaspoon",
                "key lime zest 1/4 cup",
                "strawberries 1 cups",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "(Optional but adds extra flavor). Select the 'sauté' function on the Instant Pot. Add the butter and melt. Add the steel cut oats and sauté, stirring occasionally for 5 or so minutes, until fragrant and golden.",
                    "Add the water, vanilla and cinnamon (and other ingredients as needed for flavor variations)",
                    "Cook on high pressure for 2 minutes, followed by natural pressure release (let the Instant Pot sit for 20 minutes up to 120 minutes).",
                    "Remove lid and stir in maple syrup and any other ingredients for the flavor variations.",
                    "(Optional but adds extra flavor). Heat a pan over medium heat. Add the butter and melt. Add the steel cut oats and sauté, stirring occasionally for 5 or so minutes, until fragrant and golden.",
                    "Spray the inside of your slow cooker with cooking spray or use a slow cooker liner. Transfer steel cut oats and add the water, vanilla and cinnamon (and other ingredients as needed for flavor variations)",
                    "Cover and cook on low for 6-8 hours.",
                    "Stir in maple syrup and any other ingredients for the flavor variations.",
                    "Heat a 4 quart pot over medium heat. Add the butter and melt. Add the steel cut oats and sauté, stirring occasionally for 5 or so minutes, until fragrant and golden.",
                    "Add the water, vanilla and cinnamon (and other ingredients as needed for flavor variations).",
                    "Reduce heat and simmer, stirring occasionally, for 25-35 minutes.",
                    "Remove from heat and stir in maple syrup and any other ingredients for the flavor variations.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        return self.assertEqual("4.89", self.harvester_class.ratings())

    def test_author(self):
        return self.assertEqual("Denise Bustard", self.harvester_class.author())
